#include <string>
#include <list>
#include <map>
#include <iostream>
#include "Package.h"
#include "Mapping.h"
using namespace std;

Package::Package()
	: name("INVALID"), installed(false)
{}

Package::Package(const Package& src)
{
	name=src.name;
	version=src.version;
	release=src.release;
	summary=src.summary;
	license=src.license;
	url=src.url;
	size=src.size;
	pkgsize=src.pkgsize;
	description=src.description;
	installed=src.installed;
}

Package::~Package()
{}

void Package::setname(const string& n)
{
	name=n;
}

void Package::setversion(const std::string& v)
{
	version=v;
}

void Package::setrelease(const std::string& r)
{
	release=r;
}

void Package::setsummary(const string& s)
{
	summary=s;
}

void Package::setlicense(const string& lic)
{
	map<string, int>::iterator it;
	
	it=License::mapping.find(lic);
	
	if(it==License::mapping.end())
	{
		license=License::name.size();
		License::mapping[lic]=license;
		License::name.push_back(lic);
	}
	else
		license=it->second;
}

void Package::seturl(const string& u)
{
	url=u;
}

void Package::setsize(const int s)
{
	size=s;
}

void Package::setpkgsize(const int s)
{
	pkgsize=s;
}

void Package::setdescription(const list<string>& l)
{
	description=l;
}

void Package::setinstalled(bool i)
{
	installed=i;
}

const string& Package::getname() const
{
	return name;
}

const string& Package::getversion() const
{
	return version;
}

const string& Package::getrelease() const
{
	return release;
}

const string& Package::getsummary() const
{
	return summary;
}

const string& Package::getlicense() const
{
	return License::name[license];
}

const string& Package::geturl() const
{
	return url;
}

const int Package::getsize() const
{
	return size;
}

const int Package::getpkgsize() const
{
	return pkgsize;
}

const list<string>& Package::getdescription() const
{
	return description;
}

bool Package::isinstalled() const
{
	return installed;
}

bool Package::operator==(const std::string& s) const
{
	return name==s;
}

bool Package::name_compare(const Package& n1, const Package& n2)
{
	return n1.getname() < n2.getname();
}
