#include <assert.h>
#include <string>
#include <stdexcept>
#include "TreeDir.h"
#include "Package.h"
#include "String.h"
using namespace std;

TreeDir::TreeDir()
	: name("INVALID")
{}

TreeDir::TreeDir(const string& n)
	: name(n)
{}

TreeDir::~TreeDir()
{}

void TreeDir::clear()
{
	pkgs.clear();
	dirs.clear();
}

void TreeDir::sort()
{
	dirs.sort(TreeDir::name_compare);
	pkgs.sort(Package::name_compare);
}

void TreeDir::setname(const string& n)
{
	name=n;
}

const string& TreeDir::getname() const
{
	return name;
}

TreeDir& TreeDir::operator+=(const TreeDir& t)
{
	assert(t.getname()!="INVALID");
	dirs.push_back(t);
	return dirs.back();
}

Package& TreeDir::operator+=(const Package& p)
{
	assert(p.getname()!="INVALID");
	pkgs.push_back(p);
	return pkgs.back();
}

bool TreeDir::operator==(string s) const
{
	return name==s;
}

// wstawia pakiet w odpowiednie miejsce hierarchii grup
const void TreeDir::add(const string& group, const Package& pkg)
{
	// jeeli nie ma ju gdzie zchodzi w drzewie, dodaje pakiet
	if(group.empty())
		*this+=pkg;
	else
	{
		string::const_iterator i, j;
		
		// wycicie nazwy grupy (do slasza)
		i=group.begin();
		j=find_if(i, group.end(), String::slash);
		string g(i, j);
		
		// znalezienie gazi w drzewie
		list<TreeDir>::iterator l=find(dirs.begin(), dirs.end(), g);
		if(l==dirs.end())
		{
			// dodanie, gdy ga nie istnieje
			*this+=TreeDir(g);
			l=--dirs.end();
		}
		
		// jeeli nie ma slasza w nazwie grupy, przekazanie pustej nazwy
		// grupy, jeeli jest, to wszystko co po slaszu
		if(j==group.end())
			l->add("", pkg);
		else
			l->add(string(++j, group.end()), pkg);
	}
}

TreeDir& TreeDir::find_group(const std::string& group)
{
	if(group.empty())
		return *this;
	
	string::const_iterator i, j;
		
	// wycicie nazwy grupy (do slasza)
	i=group.begin();
	j=find_if(i, group.end(), String::slash);
	string g(i, j);
		
	// znalezienie gazi w drzewie
	list<TreeDir>::iterator l=find(dirs.begin(), dirs.end(), g);
	if(l==dirs.end())
		throw invalid_argument("no such group");

	if(j==group.end())
		return *l;
	else
		return l->find_group(string(++j, group.end()));
}

Package& TreeDir::find_package(const std::string& package)
{
	list<Package>::iterator it=find(pkgs.begin(), pkgs.end(), package);
	
	if(it==pkgs.end())
		throw invalid_argument("no such package");
	else
		return *it;
}

list<TreeDir>& TreeDir::get_groups()
{
	return dirs;
}

list<Package>& TreeDir::get_packages()
{
	return pkgs;
}

bool TreeDir::name_compare(const TreeDir& n1, const TreeDir& n2)
{
	return n1.getname() < n2.getname();
}
